/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.exchange.items.task.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.StringTokenizer;
import java.util.TimeZone;

import sync4j.exchange.items.task.TaskParseException;

import sync4j.exchange.util.DateTools;
import sync4j.exchange.util.StringTools;

import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;



/**
 * Represent an Exchange Task
 * @version $Id: Task.java,v 1.23 2005/09/29 18:33:44 nichele Exp $
 **/
public class Task {

    //-------------------------------------------------------------------- Constants

    private static final String TAG_PP_ACTUAL_WORK          = "ActualWork"         ;
    private static final String TAG_PP_BILLING_INFORMATION  = "BillingInformation" ;
    private static final String TAG_PP_CATEGORIES           = "Categories"         ;
    private static final String TAG_PP_COMPANIES            = "Companies"          ;
    private static final String TAG_PP_COMPLETE             = "Complete"           ;
    private static final String TAG_PP_DATE_COMPLETED       = "DateCompleted"      ;
    private static final String TAG_PP_DUE_DATE             = "DueDate"            ;
    private static final String TAG_PP_IMPORTANCE           = "Importance"         ;
    private static final String TAG_PP_MILEAGE              = "Mileage"            ;
    private static final String TAG_PP_NOTE                 = "Body"               ;
    private static final String TAG_PP_OWNER                = "Owner"              ;
    private static final String TAG_PP_PERCENT_COMPLETE     = "PercentComplete"    ;
    private static final String TAG_PP_REMINDER_SET         = "ReminderSet"        ;
    private static final String TAG_PP_REMINDER_TIME        = "ReminderTime"       ;
    private static final String TAG_PP_SENSITIVITY          = "Sensitivity"        ;
    private static final String TAG_PP_DATE                 = "StartDate"          ;
    private static final String TAG_PP_STATUS               = "Status"             ;
    private static final String TAG_PP_SUBJECT              = "Subject"            ;
    private static final String TAG_PP_TEAM_TASK            = "TeamTask"           ;
    private static final String TAG_PP_TOTAL_WORK           = "TotalWork"          ;

    //------------------------------------------------------------- Private data
    private int     actualWork         = 0     ;
    private String  billingInformation = null  ;
    private String  categories         = null  ;
    private String  companies          = null  ;
    private boolean complete           = false ;
    private Date    date               = null  ;
    private Date    dateCompleted      = null  ;
    private Date    dueDate            = null  ;
    private String  importance         = null  ;
    private String  mileage            = null  ;
    private String  owner              = null  ;
    private String  percentComplete    = null  ;
    private boolean reminderSet        = false ;
    private Date    reminderTime       = null  ;
    private String  sensitivity        = null  ;
    private String  status             = null  ;
    private String  subject            = null  ;
    private boolean teamTask           = false ;
    private String  textDescription    = null  ;
    private int     totalWork          = 0     ;

    private String  href               = null  ;

    private String  username           = null  ;
    private String  id                 = null  ;

    private Date    createDate         = null  ;
    private Date    lastModified       = null  ;
    private char    state              = ' '   ;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Task
     *
     */
    public Task() {
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     */
    public Task (String id) {
        this.id = id;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param t client item timestamp
     */
    public Task(String id, Date t) throws TaskParseException {
        this.id           = id;
        this.lastModified = t ;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param state client item state
     */
    public Task(String id, char state) {
        this.id    = id   ;
        this.state = state;
    }

    /**
     * Creates a Task
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     */
    public Task(String id          ,
                String href        ,
                char   state       ,
                Date   lastModified) {

        this.id           = id          ;
        this.href         = href        ;
        this.state        = state       ;
        this.lastModified = lastModified;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public Task(String id, String content, Date t) throws TaskParseException {
        getPPTask(id, content, t);
    }

    //----------------------------------------------------------- Public methods
    public String getId() {
        return this.id;
    }

    public String getUsername() {
        return this.username;
    }

    public Date getDate() {
        return this.date;
    }

    public String getHref() {
        return this.href;
    }

    public Date getLastModified() {
        return this.lastModified;
    }

    public char getState() {
        return this.state;
    }

    public int getActualWork() {
        return this.actualWork;
    }

    public String getBillingInformation() {
        return this.billingInformation;
    }

    public String getCompanies() {
        return this.companies;
    }

    public String getCategories() {
        return categories;
    }

    public Date getDateCompleted() {
        return this.dateCompleted;
    }

    public Date getDueDate() {
        return dueDate;
    }

    public String getImportance() {
        return importance;
    }

    public String getMileage() {
        return mileage;
    }

    public String getOwner() {
        return this.owner;
    }

    public String getPercentComplete() {
        return percentComplete;
    }

    public boolean getReminderSet() {
        return reminderSet;
    }

    public Date getReminderTime() {
        return reminderTime;
    }

    public String getSensitivity() {
        return sensitivity;
    }

    public String getStatus() {
        return status;
    }

    public String getSubject() {
        return this.subject;
    }

    public boolean getTeamTask() {
        return this.teamTask;
    }

    public boolean isTeamTask() {
        return this.teamTask;
    }

    public String getTextDescription() {
        return this.textDescription;
    }

    public int getTotalWork() {
        return this.totalWork;
    }

    public boolean isComplete() {
        return this.complete;
    }

    public void setId(String id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setState(char state) {
        this.state = state;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public void setActualWork(int actualWork) {
        this.actualWork = actualWork;
    }

    public void setBillingInformation (String billingInformation) {
        this.billingInformation = billingInformation;
    }

    public void setCompanies(String companies) {
        this.companies = companies;
    }

    public void setComplete(boolean complete) {
        this.complete = complete;
    }

    public void setCategories(String categories) {
        this.categories = categories;
    }

    public void setDateCompleted(Date dateCompleted) {
        this.dateCompleted = dateCompleted;
    }

    public void setDueDate(Date dueDate) {
        this.dueDate = dueDate;
    }

    public void setImportance(String importance) {
        this.importance = importance;
    }

    public void setMileage(String mileage) {
        this.mileage = mileage ;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public void setPercentComplete(String percentComplete) {
        this.percentComplete = percentComplete ;
    }

    public void setReminderSet(boolean reminderSet) {
        this.reminderSet = reminderSet ;
    }

    public void setReminderTime(Date reminderTime) {
        this.reminderTime = reminderTime;
    }

    public void setSensitivity(String sensitivity) {
        this.sensitivity = sensitivity;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setTeamTask(boolean teamTask) {
        this.teamTask = teamTask;
    }

    public void setTextDescription(String textDescription) {
        this.textDescription = textDescription;
    }

    public void setTotalWork(int totalWork) {
        this.totalWork = totalWork;
    }
    /**
     * Set a Task properties from Client format content
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public void getPPTask(String id, String content, Date t)
    throws TaskParseException {

        String dt  = null;
        String tmp = null;

        this.id           = id;
        this.lastModified = t ;

        try {

            XmlParser p = new XmlParser();

            dt                      = p.getXMLInitTagValue(content, TAG_PP_DATE );
            this.date               = getDate(dt);

            tmp                     = p.getXMLInitTagValue(content, TAG_PP_ACTUAL_WORK);
            if (tmp != null && tmp.length() > 0) {
                this.actualWork = Integer.parseInt(tmp);
            }

            this.billingInformation = p.getXMLInitTagValue(content, TAG_PP_BILLING_INFORMATION);
            this.categories         = p.getXMLInitTagValue(content, TAG_PP_CATEGORIES);
            this.companies          = p.getXMLInitTagValue(content, TAG_PP_COMPANIES);
            this.dateCompleted      = getDate(p.getXMLInitTagValue(content, TAG_PP_DATE_COMPLETED ));
            this.dueDate            = getDate(p.getXMLInitTagValue(content, TAG_PP_DUE_DATE));
            this.importance         = p.getXMLInitTagValue(content, TAG_PP_IMPORTANCE);
            this.mileage            = p.getXMLInitTagValue(content, TAG_PP_MILEAGE);
            this.owner              = p.getXMLInitTagValue(content, TAG_PP_OWNER);
            this.percentComplete    = p.getXMLInitTagValue(content, TAG_PP_PERCENT_COMPLETE);

            this.complete           = sifValueToBoolean(p.getXMLInitTagValue(content, TAG_PP_COMPLETE));
            this.reminderSet        = sifValueToBoolean(p.getXMLInitTagValue(content, TAG_PP_REMINDER_SET));

            this.teamTask           = sifValueToBoolean(p.getXMLInitTagValue(content, TAG_PP_TEAM_TASK));;


            this.subject         = p.getXMLInitTagValue(content, TAG_PP_SUBJECT);
            this.textDescription = p.getXMLInitTagValue(content, TAG_PP_NOTE);


            tmp = p.getXMLInitTagValue(content, TAG_PP_REMINDER_TIME);
            if (tmp != null && !tmp.equals("")) {
                this.reminderTime = DateTools.clientDateToDate(tmp);
            }
            this.sensitivity     = p.getXMLInitTagValue(content, TAG_PP_SENSITIVITY);
            this.status          = p.getXMLInitTagValue(content, TAG_PP_STATUS);

            tmp                  = p.getXMLInitTagValue(content, TAG_PP_TOTAL_WORK);

            if (tmp != null && tmp.length() > 0) {
                this.totalWork = Integer.parseInt(tmp);
            }

        } catch (NumberFormatException e) {
            throw new TaskParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new TaskParseException(e.getMessage());
        } catch (Exception e) {
            throw new TaskParseException(e.getMessage());
        }

    }

    /**
     * Set a Task properties to Client format content
     *
     * @return content in client format
     */
    public String toXml() throws TaskParseException {

        StringBuffer xmlMsg = new StringBuffer();
        String date  = null;
        XmlParser p  = new XmlParser();

        try {


            xmlMsg.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><task>\n");

            xmlMsg.append("<" + TAG_PP_ACTUAL_WORK + ">").
                        append(String.valueOf(actualWork)).
                            append("</" +TAG_PP_ACTUAL_WORK + ">");

            if (categories != null) {
                xmlMsg.append("<" + TAG_PP_CATEGORIES + ">");
                xmlMsg.append(categories);
                xmlMsg.append("</" + TAG_PP_CATEGORIES + ">");
            }

            if (companies != null) {
                xmlMsg.append("<" + TAG_PP_COMPANIES + ">");
                xmlMsg.append(companies);
                xmlMsg.append("</" + TAG_PP_COMPANIES + ">");
            }

            xmlMsg.append("<" + TAG_PP_COMPLETE + ">").
                   append(booleanToSIFValue(complete)).
                   append("</" + TAG_PP_COMPLETE + ">");

            date = DateTools.toClientDate(this.date);

            if (date != null && date.length() > 0) {
                xmlMsg.append("<" + TAG_PP_DATE + ">");
                xmlMsg.append(date);
                xmlMsg.append("</" + TAG_PP_DATE + ">");
            } else {
                xmlMsg.append("<"  + TAG_PP_DATE + ">" +
                              "</" + TAG_PP_DATE + ">");
            }

            if (dateCompleted != null) {
                xmlMsg.append("<" + TAG_PP_DATE_COMPLETED + ">");
                xmlMsg.append(DateTools.toClientDate(dateCompleted));
                xmlMsg.append("</" + TAG_PP_DATE_COMPLETED + ">");
            }

            if (dueDate != null) {
                xmlMsg.append("<" + TAG_PP_DUE_DATE + ">");
                xmlMsg.append(DateTools.toClientDate(dueDate));
                xmlMsg.append("</" + TAG_PP_DUE_DATE + ">");
            } else {
                xmlMsg.append("<"  + TAG_PP_DUE_DATE + ">" +
                              "</" + TAG_PP_DUE_DATE + ">");
            }

            if (importance != null) {
                xmlMsg.append("<" + TAG_PP_IMPORTANCE + ">");
                xmlMsg.append(importance);
                xmlMsg.append("</" + TAG_PP_IMPORTANCE + ">");
            }

            if (mileage != null) {
                xmlMsg.append("<" + TAG_PP_MILEAGE + ">");
                xmlMsg.append(mileage);
                xmlMsg.append("</" + TAG_PP_MILEAGE + ">");
            }

            if (owner != null) {
                xmlMsg.append("<" + TAG_PP_OWNER + ">");
                xmlMsg.append(owner);
                xmlMsg.append("</" + TAG_PP_OWNER + ">");
            }

            if (percentComplete != null) {
                xmlMsg.append("<" + TAG_PP_PERCENT_COMPLETE + ">");
                xmlMsg.append(percentComplete);
                xmlMsg.append("</" + TAG_PP_PERCENT_COMPLETE + ">");
            }

            xmlMsg.append("<" + TAG_PP_REMINDER_SET + ">");
            xmlMsg.append(booleanToSIFValue(reminderSet));
            xmlMsg.append("</" + TAG_PP_REMINDER_SET + ">");

            if (reminderTime != null) {
                xmlMsg.append("<" + TAG_PP_REMINDER_TIME + ">");
                xmlMsg.append(DateTools.toClientDate(reminderTime));
                xmlMsg.append("</" + TAG_PP_REMINDER_TIME + ">");
            } else {
                xmlMsg.append("<"  + TAG_PP_REMINDER_TIME + ">" +
                              "</" + TAG_PP_REMINDER_TIME + ">");
            }

            if (sensitivity != null) {
                xmlMsg.append("<"  + TAG_PP_SENSITIVITY + ">");
                xmlMsg.append(sensitivity);
                xmlMsg.append("</" + TAG_PP_SENSITIVITY + ">");
            }

            if (status != null) {
                xmlMsg.append("<"  + TAG_PP_STATUS + ">");
                xmlMsg.append(status);
                xmlMsg.append("</" + TAG_PP_STATUS + ">");
            }

            if (subject != null) {
                xmlMsg.append("<"  + TAG_PP_SUBJECT + ">");
                xmlMsg.append(subject);
                xmlMsg.append("</" + TAG_PP_SUBJECT + ">");
            }


            xmlMsg.append("<"  + TAG_PP_TEAM_TASK + ">");
            xmlMsg.append(booleanToSIFValue(this.teamTask));
            xmlMsg.append("</" + TAG_PP_TEAM_TASK + ">");

            if (textDescription != null) {
                xmlMsg.append("<"  + TAG_PP_NOTE + ">");
                xmlMsg.append(textDescription);
                xmlMsg.append("</" + TAG_PP_NOTE + ">");
            }

            xmlMsg.append("<" + TAG_PP_TOTAL_WORK + ">").
                        append(String.valueOf(totalWork)).
                            append("</" +TAG_PP_TOTAL_WORK + ">");

            xmlMsg.append("</task>");

         } catch (Exception e) {
             throw new TaskParseException("Error composing task Device: " +
                                           e.getMessage());
         }

         return xmlMsg.toString();
     }

     private Date getDate(String sDate) throws Exception {
         if (sDate == null || sDate.equals("")) {
             return null;
         }
         SimpleDateFormat formatter = null;
         String formatUtc = "yyyyMMdd'T'HHmmss'Z'";
         formatter = new SimpleDateFormat(formatUtc);
         formatter.setTimeZone(TimeZone.getTimeZone("UTC"));

         return formatter.parse(sDate);
     }

     /**
      * Returns the SIF representation of a boolean value.
      *
      * @param value boolean
      * @return "0" if the given boolean is false, "1" otherwise
      */
     private String booleanToSIFValue(boolean value) {
         if (value) {
             return "1";
         } else {
             return "0";
         }
     }

     /**
      * If the given string is "1" returns true, otherwise returns false
      * @param value String
      * @return boolean
      */
     private boolean sifValueToBoolean(String value) {
         if (value != null && value.equals("1")) {
             return true;
         }
         return false;
    }
}
